@@ -645,12 +645,11 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
             upload = models.Upload.objects.get(pk=pk, repository=repository)
         except models.Upload.DoesNotExist as e_upload:
             # Upload has been deleted => task has started or even finished
-            try:
-                task = Task.objects.filter(
-                    name__endswith="add_and_remove",
-                    reserved_resources_record=[f"upload:{pk}"],
-                ).last()
-            except Task.DoesNotExist:
+            task = Task.objects.filter(
+                name__endswith="add_and_remove",
+                reserved_resources_record__contains=[f"shared:upload:{pk}"],
+            ).last()
+            if not task:
                 # No upload and no task for it => the upload probably never existed
                 # return 404
                 raise e_upload
@@ -671,6 +670,7 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
         with NamedTemporaryFile("ab") as temp_file:
             for chunk in chunks:
                 temp_file.write(chunk.file.read())
+                chunk.file.close()
             temp_file.flush()
 
             uploaded_file = PulpTemporaryUploadedFile.from_file(File(open(temp_file.name, "rb")))
@@ -694,8 +694,10 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
                 )
                 blob_artifact.save()
             except IntegrityError:
-                pass
-
+                ca = ContentArtifact.objects.get(content=blob, relative_path=digest)
+                if not ca.artifact:
+                    ca.artifact = artifact
+                    ca.save(update_fields=["artifact"])
             upload.delete()
 
             dispatched_task = dispatch(
@@ -838,7 +840,11 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         try:
             ca.save()
         except IntegrityError:
-            pass
+            ca = ContentArtifact.objects.get(content=manifest, relative_path=manifest.digest)
+            if not ca.artifact:
+                ca.artifact = artifact
+                ca.save(update_fields=["artifact"])
+        # both docker/oci format should contain layers, digest, media_type, size
         layers = content_data.get("layers")
         blobs = []
         for layer in layers:
