@@ -20,17 +20,18 @@ from guardian.shortcuts import get_objects_for_user
 
 from rest_framework import mixins
 from rest_framework.decorators import action
+from rest_framework.exceptions import ValidationError
 
-from pulpcore.plugin.access_policy import AccessPolicyFromDB
 from pulpcore.plugin.models import RepositoryVersion
 from pulpcore.plugin.serializers import (
     AsyncOperationResponseSerializer,
     RepositorySyncURLSerializer,
 )
 from pulpcore.plugin.models import Artifact, Content
-from pulpcore.plugin.tasking import enqueue_with_reservation
+from pulpcore.plugin.tasking import dispatch
 from pulpcore.plugin.viewsets import (
-    BaseDistributionViewSet,
+    AsyncUpdateMixin,
+    DistributionViewSet,
     BaseFilterSet,
     CharInFilter,
     ContentFilter,
@@ -224,7 +225,6 @@ class TagViewSet(ContainerContentQuerySetMixin, ReadOnlyContentViewSet):
     queryset = models.Tag.objects.all()
     serializer_class = serializers.TagSerializer
     filterset_class = TagFilter
-    permission_classes = (AccessPolicyFromDB,)
 
     DEFAULT_ACCESS_POLICY = {
         "statements": [
@@ -251,7 +251,6 @@ class ManifestViewSet(ContainerContentQuerySetMixin, ReadOnlyContentViewSet):
     queryset = models.Manifest.objects.all()
     serializer_class = serializers.ManifestSerializer
     filterset_class = ManifestFilter
-    permission_classes = (AccessPolicyFromDB,)
 
     DEFAULT_ACCESS_POLICY = {
         "statements": [
@@ -278,7 +277,6 @@ class BlobViewSet(ContainerContentQuerySetMixin, ReadOnlyContentViewSet):
     queryset = models.Blob.objects.all()
     serializer_class = serializers.BlobSerializer
     filterset_class = BlobFilter
-    permission_classes = (AccessPolicyFromDB,)
 
     DEFAULT_ACCESS_POLICY = {
         "statements": [
@@ -307,7 +305,6 @@ class ContainerRemoteViewSet(RemoteViewSet):
     endpoint_name = "container"
     queryset = models.ContainerRemote.objects.all()
     serializer_class = serializers.ContainerRemoteSerializer
-    permission_classes = (AccessPolicyFromDB,)
     queryset_filtering_required_permission = "container.view_containerremote"
 
     DEFAULT_ACCESS_POLICY = {
@@ -387,10 +384,14 @@ class TagOperationsMixin:
         manifest = serializer.validated_data["manifest"]
         tag = serializer.validated_data["tag"]
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.tag_image,
-            [repository],
-            kwargs={"manifest_pk": manifest.pk, "tag": tag, "repository_pk": repository.pk},
+            exclusive_resources=[repository],
+            kwargs={
+                "manifest_pk": str(manifest.pk),
+                "tag": tag,
+                "repository_pk": str(repository.pk),
+            },
         )
         return OperationPostponedResponse(result, request)
 
@@ -415,8 +416,10 @@ class TagOperationsMixin:
 
         tag = serializer.validated_data["tag"]
 
-        result = enqueue_with_reservation(
-            tasks.untag_image, [repository], kwargs={"tag": tag, "repository_pk": repository.pk}
+        result = dispatch(
+            tasks.untag_image,
+            exclusive_resources=[repository],
+            kwargs={"tag": tag, "repository_pk": str(repository.pk)},
         )
         return OperationPostponedResponse(result, request)
 
@@ -458,7 +461,6 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
     endpoint_name = "container"
     queryset = models.ContainerRepository.objects.all()
     serializer_class = serializers.ContainerRepositorySerializer
-    permission_classes = (AccessPolicyFromDB,)
     queryset_filtering_required_permission = "container.view_containerrepository"
     DEFAULT_ACCESS_POLICY = {
         "statements": [
@@ -556,17 +558,24 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
         Synchronizes a repository. The ``repository`` field has to be provided.
         """
         repository = self.get_object()
-        serializer = RepositorySyncURLSerializer(data=request.data, context={"request": request})
+        serializer = RepositorySyncURLSerializer(
+            data=request.data, context={"request": request, "repository_pk": pk}
+        )
 
         # Validate synchronously to return 400 errors.
         serializer.is_valid(raise_exception=True)
-        remote = serializer.validated_data.get("remote")
+        remote = serializer.validated_data.get("remote", repository.remote)
         mirror = serializer.validated_data.get("mirror")
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.synchronize,
-            [repository, remote],
-            kwargs={"remote_pk": remote.pk, "repository_pk": repository.pk, "mirror": mirror},
+            shared_resources=[remote],
+            exclusive_resources=[repository],
+            kwargs={
+                "remote_pk": str(remote.pk),
+                "repository_pk": str(repository.pk),
+                "mirror": mirror,
+            },
         )
         return OperationPostponedResponse(result, request)
 
@@ -581,23 +590,43 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
         """
         Queues a task that creates a new RepositoryVersion by adding content units.
         """
-        add_content_units = []
+        add_content_units = {}
         repository = self.get_object()
         serializer = serializers.RecursiveManageSerializer(data=request.data)
         serializer.is_valid(raise_exception=True)
 
         if "content_units" in request.data:
             for url in request.data["content_units"]:
-                content = NamedModelViewSet.get_resource(url, Content)
-                add_content_units.append(content.pk)
+                add_content_units[NamedModelViewSet.extract_pk(url)] = url
 
-        result = enqueue_with_reservation(
+            self.touch_content_units(add_content_units)
+
+        result = dispatch(
             tasks.recursive_add_content,
-            [repository],
-            kwargs={"repository_pk": repository.pk, "content_units": add_content_units},
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": repository.pk,
+                "content_units": list(add_content_units.keys()),
+            },
         )
         return OperationPostponedResponse(result, request)
 
+    def touch_content_units(self, content_units):
+        """Touch and validate referenced content units."""
+        content_units_pks = content_units.keys()
+        existing_content_units = Content.objects.filter(pk__in=content_units_pks)
+        existing_content_units.touch()
+
+        existing_content_units_pks = existing_content_units.values_list("pk", flat=True)
+        existing_content_units_pks = {str(pk) for pk in existing_content_units_pks}
+
+        missing_pks = content_units_pks - existing_content_units_pks
+        if missing_pks:
+            missing_hrefs = [content_units[pk] for pk in missing_pks]
+            raise ValidationError(
+                _("Could not find the following content units: {}").format(missing_hrefs)
+            )
+
     @extend_schema(
         description="Trigger an async task to recursively remove container content.",
         summary="Remove content",
@@ -621,12 +650,12 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
                     break
 
                 content = NamedModelViewSet.get_resource(url, Content)
-                remove_content_units.append(content.pk)
+                remove_content_units.append(str(content.pk))
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.recursive_remove_content,
-            [repository],
-            kwargs={"repository_pk": repository.pk, "content_units": remove_content_units},
+            exclusive_resources=[repository],
+            kwargs={"repository_pk": str(repository.pk), "content_units": remove_content_units},
         )
         return OperationPostponedResponse(result, request)
 
@@ -653,12 +682,13 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
         else:
             tags_to_add = tags_in_repo.filter(name__in=names)
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.recursive_add_content,
-            [repository],
+            shared_resources=[source_latest.repository],
+            exclusive_resources=[repository],
             kwargs={
-                "repository_pk": repository.pk,
-                "content_units": tags_to_add.values_list("pk", flat=True),
+                "repository_pk": str(repository.pk),
+                "content_units": [str(pk) for pk in tags_to_add.values_list("pk", flat=True)],
             },
         )
         return OperationPostponedResponse(result, request)
@@ -690,10 +720,14 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
         if media_types is not None:
             filters["media_type__in"] = media_types
         manifests_to_add = manifests_in_repo.filter(**filters)
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.recursive_add_content,
-            [repository],
-            kwargs={"repository_pk": repository.pk, "content_units": manifests_to_add},
+            shared_resources=[source_latest.repository],
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "content_units": [str(manifest.pk) for manifest in manifests_to_add],
+            },
         )
         return OperationPostponedResponse(result, request)
 
@@ -724,17 +758,20 @@ class ContainerRepositoryViewSet(TagOperationsMixin, RepositoryViewSet):
             containerfile.save()
         except IntegrityError:
             containerfile = Artifact.objects.get(sha256=containerfile.sha256)
+            containerfile.touch()
         tag = serializer.validated_data["tag"]
 
         artifacts = serializer.validated_data["artifacts"]
+        for artifact in artifacts:
+            artifact.touch()
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.build_image_from_containerfile,
-            [repository],
+            exclusive_resources=[repository],
             kwargs={
-                "containerfile_pk": containerfile.pk,
+                "containerfile_pk": str(containerfile.pk),
                 "tag": tag,
-                "repository_pk": repository.pk,
+                "repository_pk": str(repository.pk),
                 "artifacts": artifacts,
             },
         )
@@ -747,7 +784,6 @@ class ContainerRepositoryVersionViewSet(RepositoryVersionQuerySetMixin, Reposito
     """
 
     parent_viewset = ContainerRepositoryViewSet
-    permission_classes = (AccessPolicyFromDB,)
     queryset_filtering_required_repo_permission = "container.view_containerrepository"
 
     DEFAULT_ACCESS_POLICY = {
@@ -781,11 +817,22 @@ class ContainerRepositoryVersionViewSet(RepositoryVersionQuerySetMixin, Reposito
                     "has_repo_attr_model_or_obj_perms:container.view_containerrepository",
                 ],
             },
+            {
+                "action": ["repair"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": [
+                    "has_repo_attr_model_or_obj_perms:container.sync_containerrepository",
+                    "has_repo_attr_model_or_obj_perms:container.view_containerrepository",
+                ],
+            },
         ],
     }
 
 
-class ContainerPushRepositoryViewSet(TagOperationsMixin, ReadOnlyRepositoryViewSet):
+class ContainerPushRepositoryViewSet(
+    TagOperationsMixin, ReadOnlyRepositoryViewSet, AsyncUpdateMixin
+):
     """
     ViewSet for a container push repository.
 
@@ -821,6 +868,15 @@ class ContainerPushRepositoryViewSet(TagOperationsMixin, ReadOnlyRepositoryViewS
                     "has_namespace_or_obj_perms:container.view_containerpushrepository",
                 ],
             },
+            {
+                "action": ["update", "partial_update"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": [
+                    "has_namespace_or_obj_perms:container.change_containerpushrepository",
+                    "has_namespace_or_obj_perms:container.view_containerpushrepository",
+                ],
+            },
         ],
         "permissions_assignment": [
             {
@@ -832,6 +888,7 @@ class ContainerPushRepositoryViewSet(TagOperationsMixin, ReadOnlyRepositoryViewS
                 "permissions": [
                     "container.view_containerpushrepository",
                     "container.modify_content_containerpushrepository",
+                    "container.change_containerpushrepository",
                 ],
             },
             {
@@ -843,6 +900,7 @@ class ContainerPushRepositoryViewSet(TagOperationsMixin, ReadOnlyRepositoryViewS
                 "permissions": [
                     "container.view_containerpushrepository",
                     "container.modify_content_containerpushrepository",
+                    "container.change_containerpushrepository",
                 ],
             },
             {
@@ -883,10 +941,13 @@ class ContainerPushRepositoryViewSet(TagOperationsMixin, ReadOnlyRepositoryViewS
         content_units_to_remove = list(serializer.validated_data["tags_pks"])
         content_units_to_remove.append(serializer.validated_data["manifest"].pk)
 
-        result = enqueue_with_reservation(
+        result = dispatch(
             tasks.recursive_remove_content,
-            [repository],
-            kwargs={"repository_pk": repository.pk, "content_units": content_units_to_remove},
+            exclusive_resources=[repository],
+            kwargs={
+                "repository_pk": str(repository.pk),
+                "content_units": [str(pk) for pk in content_units_to_remove],
+            },
         )
         return OperationPostponedResponse(result, request)
 
@@ -928,7 +989,6 @@ class ContainerPushRepositoryVersionViewSet(
     """
 
     parent_viewset = ContainerPushRepositoryViewSet
-    permission_classes = (AccessPolicyFromDB,)
     queryset_filtering_required_repo_permission = "container.view_containerpushrepository"
 
     DEFAULT_ACCESS_POLICY = {
@@ -948,7 +1008,7 @@ class ContainerPushRepositoryVersionViewSet(
     }
 
 
-class ContainerDistributionViewSet(BaseDistributionViewSet):
+class ContainerDistributionViewSet(DistributionViewSet):
     """
     The Container Distribution will serve the latest version of a Repository if
     ``repository`` is specified. The Container Distribution will serve a specific
@@ -979,7 +1039,19 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
                 "action": ["create"],
                 "principal": "authenticated",
                 "effect": "allow",
-                "condition": "has_model_perms:container.add_containerdistribution",
+                "condition": "has_namespace_model_perms",
+            },
+            {
+                "action": ["create"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": "has_namespace_perms:container.add_containerdistribution",
+            },
+            {
+                "action": ["create"],
+                "principal": "authenticated",
+                "effect": "allow",
+                "condition": "namespace_is_username",
             },
             {
                 "action": ["retrieve"],
@@ -993,7 +1065,7 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
                 "action": ["pull"],
                 "principal": "*",
                 "effect": "allow",
-                "condition": [
+                "condition_expression": [
                     "not is_private",
                 ],
             },
@@ -1010,7 +1082,7 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": [
-                    "has_model_or_obj_perms:container.change_containerdistribution",
+                    "has_namespace_or_obj_perms:container.change_containerdistribution",
                     "has_namespace_or_obj_perms:container.view_containerdistribution",
                 ],
             },
@@ -1065,6 +1137,7 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
                 "permissions": [
                     "container.view_containerpushrepository",
                     "container.modify_content_containerpushrepository",
+                    "container.change_containerpushrepository",
                 ],
             },
             {
@@ -1087,6 +1160,7 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
                 "permissions": [
                     "container.view_containerpushrepository",
                     "container.modify_content_containerpushrepository",
+                    "container.change_containerpushrepository",
                 ],
             },
             {
@@ -1139,16 +1213,16 @@ class ContainerDistributionViewSet(BaseDistributionViewSet):
         distribution = self.get_object()
         reservations = [distribution]
         instance_ids = [
-            (distribution.pk, "container", "ContainerDistributionSerializer"),
+            (str(distribution.pk), "container", "ContainerDistributionSerializer"),
         ]
         if distribution.repository and distribution.repository.cast().PUSH_ENABLED:
             reservations.append(distribution.repository)
             instance_ids.append(
-                (distribution.repository.pk, "container", "ContainerPushRepositorySerializer"),
+                (str(distribution.repository.pk), "container", "ContainerPushRepositorySerializer"),
             )
 
-        async_result = enqueue_with_reservation(
-            tasks.general_multi_delete, reservations, args=(instance_ids,)
+        async_result = dispatch(
+            tasks.general_multi_delete, exclusive_resources=reservations, args=(instance_ids,)
         )
         return OperationPostponedResponse(async_result, request)
 
@@ -1246,6 +1320,7 @@ class ContainerNamespaceViewSet(
                     "container.namespace_change_containerdistribution",
                     "container.namespace_view_containerpushrepository",
                     "container.namespace_modify_content_containerpushrepository",
+                    "container.namespace_change_containerpushrepository",
                 ],
             },
             {
@@ -1264,6 +1339,7 @@ class ContainerNamespaceViewSet(
                     "container.namespace_change_containerdistribution",
                     "container.namespace_view_containerpushrepository",
                     "container.namespace_modify_content_containerpushrepository",
+                    "container.namespace_change_containerpushrepository",
                 ],
             },
             {
@@ -1299,19 +1375,23 @@ class ContainerNamespaceViewSet(
 
             reservations.append(distribution)
             instance_ids.append(
-                (distribution.pk, "container", "ContainerDistributionSerializer"),
+                (str(distribution.pk), "container", "ContainerDistributionSerializer"),
             )
             if distribution.repository and distribution.repository.cast().PUSH_ENABLED:
                 reservations.append(distribution.repository)
                 instance_ids.append(
-                    (distribution.repository.pk, "container", "ContainerPushRepositorySerializer"),
+                    (
+                        str(distribution.repository.pk),
+                        "container",
+                        "ContainerPushRepositorySerializer",
+                    ),
                 )
 
         reservations.append(namespace)
         instance_ids.append(
-            (namespace.pk, "container", "ContainerNamespaceSerializer"),
+            (str(namespace.pk), "container", "ContainerNamespaceSerializer"),
         )
-        async_result = enqueue_with_reservation(
-            tasks.general_multi_delete, reservations, args=(instance_ids,)
+        async_result = dispatch(
+            tasks.general_multi_delete, exclusive_resources=reservations, args=(instance_ids,)
         )
         return OperationPostponedResponse(async_result, request)
