@@ -8,9 +8,8 @@ import unittest
 from urllib.parse import urljoin, urlparse
 
 from pulp_smash import api, cli, config, exceptions
-from pulp_smash.pulp3.bindings import monitor_task
+from pulp_smash.pulp3.bindings import delete_orphans, monitor_task
 from pulp_smash.pulp3.utils import (
-    delete_orphans,
     get_content,
     gen_distribution,
     gen_repo,
@@ -21,16 +20,16 @@ from pulp_container.tests.functional.utils import (
     core_client,
     gen_container_client,
     gen_container_remote,
-    get_docker_hub_remote_blobsums,
+    get_blobsums_from_remote_registry,
     BearerTokenAuth,
     AuthenticationHeaderQueries,
 )
 from pulp_container.tests.functional.constants import (
     CONTAINER_CONTENT_NAME,
-    REPO_UPSTREAM_NAME,
-    REPO_UPSTREAM_TAG,
+    REGISTRY_V2_REPO_HELLO_WORLD,
+    PULP_HELLO_WORLD_LINUX_TAG,
 )
-from pulp_container.constants import MEDIA_TYPE
+from pulp_container.constants import EMPTY_BLOB, MEDIA_TYPE
 
 from pulpcore.client.pulp_container import (
     ContainerContainerDistribution,
@@ -140,7 +139,7 @@ class PullContentTestCase(unittest.TestCase):
         # Assert that at least one layer is synced from remote:latest
         # and the checksum matched with remote
         self.assertTrue(
-            any([result["blobSum"] in checksums for result in get_docker_hub_remote_blobsums()]),
+            any([checksum in checksums for checksum in get_blobsums_from_remote_registry()]),
             "Cannot find a matching layer on remote registry.",
         )
 
@@ -184,7 +183,40 @@ class PullContentTestCase(unittest.TestCase):
         # the header digest should be equal to the SHA256 hash computed from
         # a manifest without signatures
         computed_digest = hashlib.sha256(manifest_string).hexdigest()
-        self.assertEqual(computed_digest, header_digest, "The manifest digests are not equal")
+        self.assertEqual(
+            computed_digest, header_digest.split(":")[1], "The manifest digests are not equal"
+        )
+
+    def test_create_empty_blob_on_the_fly(self):
+        """
+        Test if empty blob getscreated and served on the fly.
+        """
+        blob_path = "/v2/{}/blobs/{}".format(self.distribution_with_repo.base_path, EMPTY_BLOB)
+        empty_blob_url = urljoin(self.cfg.get_base_url(), blob_path)
+
+        if TOKEN_AUTH_DISABLED:
+            auth = ()
+        else:
+            with self.assertRaises(requests.HTTPError) as cm:
+                requests.get(empty_blob_url).raise_for_status()
+
+            content_response = cm.exception.response
+            self.assertEqual(content_response.status_code, 401)
+
+            authenticate_header = content_response.headers["Www-Authenticate"]
+            queries = AuthenticationHeaderQueries(authenticate_header)
+            content_response = requests.get(
+                queries.realm, params={"service": queries.service, "scope": queries.scope}
+            )
+            content_response.raise_for_status()
+            auth = BearerTokenAuth(content_response.json()["token"])
+        content_response = requests.get(empty_blob_url, auth=auth)
+        content_response.raise_for_status()
+        # calculate digest of the payload
+        digest = hashlib.sha256(content_response.content).hexdigest()
+        # compare with the digest returned in the response header
+        header_digest = content_response.headers["docker-content-digest"].split(":")[1]
+        self.assertEqual(digest, header_digest)
 
     def test_pull_image_from_repository(self):
         """Verify that a client can pull the image from Pulp.
@@ -204,11 +236,11 @@ class PullContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_from_repository_version(self):
         """Verify that a client can pull the image from Pulp.
@@ -228,11 +260,11 @@ class PullContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_with_tag(self):
         """Verify that a client can pull the image from Pulp with a tag.
@@ -248,16 +280,18 @@ class PullContentTestCase(unittest.TestCase):
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
-            + REPO_UPSTREAM_TAG
+            + PULP_HELLO_WORLD_LINUX_TAG
         )
 
         registry.pull(local_url)
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
-        self.teardown_cleanups.append((registry.rmi, REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG))
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        self.teardown_cleanups.append(
+            (registry.rmi, REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        )
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
 
@@ -376,7 +410,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         # Assert that at least one layer is synced from remote:latest
         # and the checksum matched with remote
         self.assertTrue(
-            any([result["blobSum"] in checksums for result in get_docker_hub_remote_blobsums()]),
+            any([checksum in checksums for checksum in get_blobsums_from_remote_registry()]),
             "Cannot find a matching layer on remote registry.",
         )
 
@@ -399,15 +433,15 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
 
         new_artifact_count = self.artifacts_api.list().count
         self.assertGreater(new_artifact_count, self.artifact_count)
 
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_from_repository_version(self):
         """Verify that a client can pull the image from Pulp (on-demand).
@@ -427,11 +461,11 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME)
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD)
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
-        registry.rmi(REPO_UPSTREAM_NAME)
+        registry.rmi(REGISTRY_V2_REPO_HELLO_WORLD)
 
     def test_pull_image_with_tag(self):
         """Verify that a client can pull the image from Pulp with a tag (on-demand).
@@ -447,15 +481,17 @@ class PullOnDemandContentTestCase(unittest.TestCase):
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
-            + REPO_UPSTREAM_TAG
+            + PULP_HELLO_WORLD_LINUX_TAG
         )
 
         registry.pull(local_url)
         self.teardown_cleanups.append((registry.rmi, local_url))
         local_image = registry.inspect(local_url)
 
-        registry.pull(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
-        self.teardown_cleanups.append((registry.rmi, REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG))
-        remote_image = registry.inspect(REPO_UPSTREAM_NAME + REPO_UPSTREAM_TAG)
+        registry.pull(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        self.teardown_cleanups.append(
+            (registry.rmi, REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
+        )
+        remote_image = registry.inspect(REGISTRY_V2_REPO_HELLO_WORLD + PULP_HELLO_WORLD_LINUX_TAG)
 
         self.assertEqual(local_image[0]["Id"], remote_image[0]["Id"])
